package de.mcs.tools.sps.holtec;

import static org.junit.Assert.fail;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

import java.util.ArrayList;
import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.EmulatorInternals;
import de.mcs.tools.sps.EmulatorOutput;
import de.mcs.tools.sps.SPSCommand;
import de.mcs.tools.sps.SPSCommandData;
import de.mcs.tools.sps.SPSEmulator;
import de.mcs.tools.sps.TOutputCallback;
import de.mcs.tools.sps.exceptions.WrongProgramSizeException;

class TestHoltecEmualtor implements TOutputCallback {

  private SPSEmulator holtectEmulator;

  @BeforeEach
  void setUp() throws Exception {
    holtectEmulator = new HoltecEmulator();
    holtectEmulator.addCallback(this);
  }

  @Test
  void testOutputAllCommands() {
    System.out.println("command list holtect");
    List<SPSCommand> commands = holtectEmulator.getCommands();
    commands.forEach(c -> {
      List<SPSCommandData> commandDatas = c.getCommandDatas();
      System.out.printf("Command %s(%s):", c.getMnemonic(), c.getName());
      commandDatas.forEach(cd -> {
        System.out.printf("%s(%s) ,", cd.getMnemonic(), cd.getName());
      });
      System.out.println();
    });
  }

  @Test
  void testProgramSize() throws WrongProgramSizeException {
    assertEquals(128, holtectEmulator.getProgramSize());
    byte[] prgMemory = new byte[128];
    holtectEmulator.loadProgram(prgMemory);
  }

  @Test
  void testWrongProgramSize() throws WrongProgramSizeException {
    byte[] prgMemory = new byte[129];
    Throwable exception = assertThrows(WrongProgramSizeException.class, () -> holtectEmulator.loadProgram(prgMemory));
  }

  @Test
  void testCommandArray() {
    List<SPSCommand> commands = holtectEmulator.getCommands();
    assertEquals(15, commands.size());
    for (SPSCommand spsCommand : commands) {
      switch (spsCommand.getCommandByte()) {
      case 0x00:
        assertEquals(1, spsCommand.getCommandDatas().size());
        break;
      case 0x10:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x20:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x30:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x40:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case 0x50:
        assertEquals(9, spsCommand.getCommandDatas().size());
        break;
      case 0x60:
        assertEquals(10, spsCommand.getCommandDatas().size());
        break;
      case 0x70:
        assertEquals(10, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0x80:
        assertEquals(8, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0x90:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xA0:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xB0:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xC0:
        assertEquals(15, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xD0:
        assertEquals(16, spsCommand.getCommandDatas().size());
        break;
      case (byte) 0xE0:
        assertEquals(1, spsCommand.getCommandDatas().size());
        break;
      default:
        fail(String.format("unknow command %s", spsCommand.getName()));
        break;
      }
    }
  }

  @Test
  void testNOP() throws WrongProgramSizeException {
    outputList.clear();
    internalList.clear();
    byte[] prg = new byte[] { 0x00, 0x00, 0x00 };
    holtectEmulator.loadProgram(prg);
    holtectEmulator.startProgram(true);
    holtectEmulator.nextStep();
    holtectEmulator.nextStep();
    holtectEmulator.nextStep();

    assertEquals(4, outputList.size());
    assertEquals(4, internalList.size());
  }

  List<EmulatorOutput> outputList = new ArrayList();
  List<EmulatorInternals> internalList = new ArrayList<>();

  @Override
  public void onOutput(EmulatorOutput output) {
    outputList.add(output);
    internalList.add(holtectEmulator.getInternals());
  }
}
